<?php
defined('_JEXEC') or die('Restricted access');

class plgHikashoppaymentComnpay extends hikashopPaymentPlugin
{
	var $version = '1.1';
	var $accepted_currencies = array( "EUR", "USD" ); 
	var $multiple = true; // Multiple plugin configurations. It should usually be set to true
	var $name = 'comnpay'; //Payment plugin name (the name of the PHP file)
	//This array contains the specific configuration needed (Back end > payment plugin edition)
	var $pluginConfig = array(
		'tpe' => array('TPE Number', 'input','DEMO'),
		'key' => array('Secret Key', 'input','DEMO'),
		'url_homologation' => array('URL Homologation','input','https://secure-homologation.comnpay.com'),
		'url_production'=>array('URL Production','input','https://secure.comnpay.com'),
		'debug' => array('DEBUG', 'boolean','0'),
		'plateforme' => array('Plateforme', 'list',array(
			'homologation'=>'Homologation',
			'production'=>'Production')
		),
		'cancel_url' => array('CANCEL_URL', 'input'),
		'return_url' => array('RETURN_URL', 'input'),
		'type_transaction' => array('Transaction type', 'list',array(
			'D'=>'D&eacute;bit imm&eacute;diat'
			//'PA'=>'Pr&eacute; Autorisation'
			)
		),
		'verified_status' => array('VERIFIED_STATUS', 'orderstatus', 'confirmed'),
		'invalid_status' => array('INVALID_STATUS', 'orderstatus', 'cancelled'),
	);

	/*To set the specific configuration (back end) default values (see $pluginConfig array)*/
	function getPaymentDefaultValues(&$element) {
		$element->payment_name = 'comNpay';
		$element->payment_description = 'Pay by credit card through our secure gateway comNpay';
		$element->payment_images = 'MasterCard,VISA,Credit_card,American_Express';
		$element->payment_params->tpe = 'DEMO';
		$element->payment_params->key = 'DEMO';
		$element->payment_params->url_homologation = 'https://secure-homologation.comnpay.com';
		$element->payment_params->url_production = 'https://secure.comnpay.com';
		$element->payment_params->invalid_status = 'cancelled';
		$element->payment_params->type_transaction = 'D';
		$element->payment_params->pending_status = 'created';
		$element->payment_params->verified_status = 'confirmed';
	}

	/*This function is called at the beginning of the checkout(check the empty fields in the comnpay config)*/
	function onBeforeOrderCreate(&$order,&$do){
		if(parent::onBeforeOrderCreate($order, $do) === true)
			return true;

		if(empty($this->payment_params->tpe) || empty($this->payment_params->key)) {
			$this->app->enqueueMessage('You have to configure comNpay settings');
			$do = false;
		}
	}

	//*This function is called at the end of the checkout. That's the function which should display your payment gateway redirection form with the data from HikaShop */
	function onAfterOrderConfirm(&$order,&$methods,$method_id) {
		parent::onAfterOrderConfirm($order,$methods,$method_id);
		$config = JFactory::getConfig();
		if(!empty($this->payment_params->cancel_url))
			$nok_url = $this->payment_params->cancel_url;
		else	
			$nok_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order->order_id.$this->url_itemid;
		if(!empty($this->payment_params->return_url))
			$ok_url = $this->payment_params->return_url;
		else
			$ok_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$order->order_id.$this->url_itemid;
		
		$ipn_url = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&amp;notif_payment='.$this->name.'&tmpl=component';
				
		$this->vars = array(
			'montant'=>round($order->cart->full_total->prices[0]->price_value_with_tax,2),
			'idTPE'=>$this->payment_params->tpe,
			'idTransaction'=>time().'-'.$order->order_id,
			'devise'=>$this->currency->currency_code,
			'lang'=>$this->locale,
			'nom_produit'=>$config->get( 'sitename' ),
			'source'=>$_SERVER['SERVER_NAME'],
			'urlRetourOK'=>$ok_url,
			'urlRetourNOK'=>$nok_url,
			'typeTr'=>$this->payment_params->type_transaction,
			'urlIPN'=>$ipn_url,
			'extension'=>'hikashop-'.$this->version,
			'key'=>$this->payment_params->key,
			);

		$comnpayWithKey = base64_encode(implode("|", $this->vars));
		unset($this->vars['key']);
		$this->vars['sec'] = hash("sha512",$comnpayWithKey);
	
		$this->showPage('end'); //on renvoie vers la page qui contient le formulaire
		return true;
	}

    /*After submiting the plateform payment form, this is where the website will receive the response information from the IPN comnpay gateway and then validate or not the order.*/
	function onPaymentNotification(&$statuses) {
		$feedback = $this->getfeedback();

		$transaction_id = explode('-',$feedback['idTransaction']);
		$order_id = $transaction_id[1];
		$order = $this->getOrder($order_id);
		$this->loadPaymentParams($order);
		if(empty($this->payment_params))
			return false;
		$this->loadOrderData($order);
		if($this->payment_params->debug)
			echo print_r($order);
		if(!$this->validSec($feedback,$this->payment_params->key))  {
			header("Status: 400 Bad Request", false, 400);
			if($this->payment_params->debug)
				echo "Impossible de valider la cl secrte";
			exit();
			return FALSE;
		}
		else {
			if($this->payment_params->debug)
			echo "Cl Secrte valide";
			if($feedback['result']=="OK") 
				$order_status = $this->payment_params->verified_status;
			else
				$order_status = $this->payment_params->invalid_status;
		}
		$this->modifyOrder($order_id, $order_status, true,true);
		return true;
	}
	
	/*This function return the POST parameters from IPN comNpay gateway */
	function getfeedback() {
		$feedback = $_POST;
		unset($feedback['hikashop_front_end_main']); // generate by hikashop
		unset($feedback['view']); //idem
		return $feedback;
	}
	/*This function valid a secret key passed as a parameter*/
	function validSec($values, $secret_key){
		if (isset($values['sec']) && $values['sec'] != ""){
			$sec = $values['sec'];
			unset($values['sec']);
			return strtoupper(hash("sha512", base64_encode(implode("|",$values)."|".$secret_key))) == strtoupper($sec);
		}
		else{
			return false;
		}
	}
	
	
}

?>